import os.path
import google.auth

from google.oauth2.credentials import Credentials
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from googleapiclient.http import MediaFileUpload
from google.oauth2 import service_account

# If modifying these scopes, delete the file credentials.
SCOPES = ["https://www.googleapis.com/auth/drive"]
credential_path = '../Supplementary_data/DriveCredentials/credentials.json'


def create_token():
    ''''
        credential: provide the json creditials you would get from google service.
    '''
    creds = None
    creds = service_account.Credentials.from_service_account_file(credential_path, scopes=SCOPES)
    return creds

def list_gdrive():
    '''
    List the 10 recent files from the google drive
    '''
    creds = create_token()
    try:
        service = build("drive", "v3", credentials=creds)
      
        results = (service.files().list(pageSize=20, fields="nextPageToken, files(id, name)").execute())
        items = results.get("files", [])

        if not items:
            print("No files found.")
            return
        print("Files:")
        for item in items:
            print(f"{item['name']} ({item['id']})")
    except HttpError as error:
        print(f"An error occurred: {error}")


def upload_to_gdrive(file_path=None):
    '''
        Uploading files to google drive
    '''
    creds = create_token()
    try:
        # create drive api client
        service = build("drive", "v3", credentials=creds)
        folder_path = '../Supplementary_data/DriveCredentials/googledrive_id.txt'
        #read the first line of the file
        folder_id = open(folder_path, "r").readline()

        file_metadata = {"name": file_path, "parents": [folder_id]}
        media = MediaFileUpload(file_path, resumable=True)
        # pylint: disable=maybe-no-member
        file = (service.files().create(body=file_metadata, media_body=media).execute())
        print('File Uploaded successful')
    except HttpError as error:
        print(f"An error occurred: {error}")
        file = None
    return


def delete_to_gdrive(file_id=None):
    '''
        deleting file from google drive
    '''
    creds = create_token()
    try:
        # create drive api client
        service = build("drive", "v3", credentials=creds)

        # pylint: disable=maybe-no-member
        response = service.files().delete(fileId=file_id).execute()
        print('File deleted successful')
    except HttpError as error:
        print(f"An error occurred: {error}")
        file = None
    return


"""
Function for defining an area of interest using either a point and buffer or a vector file.
"""

import geopandas as gpd
from geojson import Feature, FeatureCollection, Point
from shapely.geometry import box


def define_area(
    lat: float = None,
    lon: float = None,
    buffer: float = None,
    lat_buffer: float = None,
    lon_buffer: float = None,
    vector_path: str = None,
) -> FeatureCollection:
    """
    Define an area of interest using either a point and buffer or separate latitude and longitude buffers, or a vector.

    Parameters:
    -----------
    lat : float, optional
        The latitude of the center point of the area of interest.
    lon : float, optional
        The longitude of the center point of the area of interest.
    buffer : float, optional
        The buffer around the center point, in degrees. This is used if separate latitude and longitude buffers are not provided.
    lat_buffer : float, optional
        The buffer around the center point, extending along the latitude, in degrees.
    lon_buffer : float, optional
        The buffer around the center point, extending along the longitude, in degrees.
    vector_path : str, optional
        The path to a vector defining the area of interest.

    Returns:
    --------
    feature_collection : FeatureCollection
        A GeoJSON feature collection representing the area of interest.
    """
    # Check if both buffer and separate lat/lon buffers are specified
    if buffer is not None and (lat_buffer is not None or lon_buffer is not None):
        raise ValueError("Specify either buffer or separate lat_buffer and lon_buffer, not both.")

    # Check if either lat_buffer or lon_buffer is provided without the other
    if (lat_buffer is not None and lon_buffer is None) or (
        lat_buffer is None and lon_buffer is not None
    ):
        raise ValueError("Both lat_buffer and lon_buffer must be provided together.")

    # Ensure buffer values are positive
    # if negative values are provided for buffer, lat_buffer, or lon_buffer,
    # they will be converted to their absolute values without raising an error.
    if buffer is not None:
        buffer = abs(buffer)
    if lat_buffer is not None:
        lat_buffer = abs(lat_buffer)
    if lon_buffer is not None:
        lon_buffer = abs(lon_buffer)

    # Define area using point and buffer
    if lat is not None and lon is not None:
        if buffer is not None and (lat_buffer is None or lon_buffer is None):
            lat_buffer = lon_buffer = buffer

        if lat_buffer is not None and lon_buffer is not None:
            lat_range = (lat - lat_buffer, lat + lat_buffer)
            lon_range = (lon - lon_buffer, lon + lon_buffer)
            box_geom = box(min(lon_range), min(lat_range), max(lon_range), max(lat_range))
            aoi = gpd.GeoDataFrame(geometry=[box_geom], crs="EPSG:4326")
        else:
            aoi = gpd.GeoDataFrame(geometry=[Point(lon, lat).buffer(buffer)], crs="EPSG:4326")

    # Define area using vector
    elif vector_path is not None:
        aoi = gpd.read_file(vector_path).to_crs("EPSG:4326")
    # If neither option is provided, raise an error
    else:
        raise ValueError("Either lat/lon/buffer or vector_path must be provided.")

    # Convert the GeoDataFrame to a GeoJSON FeatureCollection
    features = [
        Feature(geometry=row["geometry"], properties=row.drop("geometry").to_dict())
        for _, row in aoi.iterrows()
    ]
    feature_collection = FeatureCollection(features)

    return feature_collection



import collections

import numpy as np
import pandas as pd
import requests
import rioxarray
import xarray as xr

from deafrica_tools.spatial import add_geobox

BASE_URL = "https://data.apps.fao.org/gismgr/api/v2/catalog/workspaces/WAPOR-3/mapsets"


def get_WaPORv3_info(url: str, info: str | list[str]) -> pd.DataFrame:
    """
    Get information on WaPOR v3 data. WaPOR v3 variables are stored in `mapsets`,
    which in turn contain `rasters` that contain the data for a particular date or period.

    Parameters
    ----------
    url : str
        URL to get information from
    info : str | list[str]
        Attribute of a mapset

    Returns
    -------
    pd.DataFrame
        A table of the mapset attributes found.
    """
    if isinstance(info, str):
        info = [info]
    elif isinstance(info, list):
        info = [str(i) for i in info]
    else:
        raise TypeError("'info' must be a list or string.")

    data = {"links": [{"rel": "next", "href": url}]}

    output_dict = collections.defaultdict(list)
    while "next" in [x["rel"] for x in data["links"]]:
        url_ = [x["href"] for x in data["links"] if x["rel"] == "next"][0]
        response = requests.get(url_)
        response.raise_for_status()
        data = response.json()["response"]
        for item in data["items"]:
            if info == ["all"]:
                for key in list(item.keys()):
                    output_dict[key].append(item[key])
            else:
                for key in info:
                    output_dict[key].append(item[key])

    output_df = pd.DataFrame(output_dict)

    if "code" in output_df.columns:
        output_df.sort_values("code", inplace=True)
        output_df.reset_index(drop=True, inplace=True)
    return output_df


def get_all_WaPORv3_mapsets() -> pd.DataFrame:
    """
    List all the available WaPOR v3 mapset codes and descriptions.

    Returns
    -------
    pd.DataFrame
        A table of all the available WaPOR v3 mapset codes and descriptions.
    """
    output_df = get_WaPORv3_info(url=BASE_URL, info=["code", "caption"])
    output_df = output_df.rename(columns={"code": "Mapset Code", "caption": "Mapset Description"})

    return output_df


def load_wapor_ds(filename: str, variable: str) -> xr.Dataset:
    """
    Load the netCDF downloaded using `wapordl.wapor_map` as a
    xarray Dataset. 
    Note: Only works for netcdf files, this is because when loading the WAPOR TIFF 
    files, the start_date and end_date attributes are only loaded for the first band.

    Parameters
    ----------
    filename : str
        File path of the netCDF downloaded using `wapordl.wapor_map`
    variable : str
        Name of the WAPOR version 3 mapset downloaded

    Returns
    -------
    xr.Dataset
        Dataset containing the WAPOR version 3 mapset data downloaded.
    """
    # Load the file.
    if filename.endswith(".tif"):
        raise ValueError("Please set extension='.nc' when downloading data using `wapor_map`")
    elif filename.endswith(".nc"):
        ds = rioxarray.open_rasterio(filename).squeeze(dim="band").drop_vars("band")
        
    # Store the crs in the spatial_ref coordinate
    if "spatial_ref" in ds.coords:
        crs_attrs = ds["spatial_ref"].attrs
        ds = add_geobox(ds.drop_vars("spatial_ref"), crs=ds.rio.crs)
    elif "crs" in ds.coords:
        crs_attrs = ds["crs"].attrs
        ds = add_geobox(ds.drop_vars("crs"), crs=ds.rio.crs)
    elif "transverse_mercator" in ds.coords:
        crs_attrs = ds["transverse_mercator"].attrs
        ds = add_geobox(ds.drop_vars("transverse_mercator"), crs=ds.rio.crs)

    ds["spatial_ref"].attrs.update(crs_attrs)

    # Get the bands names.
    bands = [var for var in list(ds.variables) if var not in list(ds.coords)]
    # Assign the correct time coordinate to each band.
    # This is the start date in the attributes.
    da_list = [
        ds[band]
        .assign_coords(time=np.datetime64(ds[band].attrs["start_date"], "ns"))
        .expand_dims(time=1)
        .rename(variable)
        for band in bands
    ]

    # Merge the DataArrays
    da = xr.concat(da_list, dim="time")
    
    # Rescale data
    xr.set_options(keep_attrs=True)
    if "scale_factor" in da.attrs:
        da = da * da.attrs['scale_factor'] + da.attrs['add_offset']

    # Edit the attributes
    if len(da.time) > 1:
        attrs_to_drop = ["start_date", "end_date", "number_of_days"]
        for attr in attrs_to_drop:
            del da.attrs[attr]

    return da.to_dataset(promote_attrs=True).sortby("time", ascending=True)
